@@ -18,21 +18,16 @@ class Hook(Configurable, Openable):
         super().__init__(name, config, host)
         self.virtual = virtual
 
-    async def start(self):
-        """
-        Perform any setup tasks.
-        """
-
-    async def stop(self):
-        """
-        Perform any teardown tasks.
-        """
-
     def on_load(self):
         """
         Perform any additional one-time setup that requires other plugs or hooks to be loaded.
         """
 
+    def on_ready(self):
+        """
+        Perform any post-startup tasks once all hooks and plugs are ready.
+        """
+
     async def channel_migrate(self, old, new):
         """
         Move any private data between channels on admin request.  This is intended to cover data
