@@ -4,6 +4,11 @@ Web-based management UI for a host instance.
 Dependencies:
     :class:`.WebHook` with templating
 
+    Extra name: ``webui``
+
+    `Docutils <https://docutils.sourceforge.io>`_:
+        Used to render module documentation if available.
+
 Config:
     route (str):
         Path to expose the UI pages under.
@@ -20,8 +25,9 @@ This is a simple control panel to add or update plugs and hooks in a running sys
     HTTP authentication, or use a service layer like oauth2_proxy.
 """
 
-from asyncio import gather
+from asyncio import ensure_future, gather
 from collections import defaultdict
+from datetime import datetime, timedelta
 from importlib import import_module
 from inspect import cleandoc
 import json
@@ -37,6 +43,11 @@ except ImportError:
 
 from aiohttp import web
 
+try:
+    from docutils.core import publish_parts
+except ImportError:
+    publish_parts = None
+
 import immp
 from immp.hook.runner import RunnerHook
 from immp.hook.web import WebHook
@@ -45,9 +56,16 @@ from immp.hook.web import WebHook
 log = logging.getLogger(__name__)
 
 
-def _module_doc(obj):
+def _render_module_doc(obj):
     doc = import_module(obj.__module__).__doc__
-    return cleandoc(doc) if doc else None
+    if not doc:
+        return (None, None)
+    doc = re.sub(r":[a-z]+:`\.?(.+?)`", r"``\1``", cleandoc(doc))
+    html = None
+    if publish_parts:
+        parts = publish_parts(doc, writer_name="html", settings_overrides={"report_level": 4})
+        html = parts["body"]
+    return (doc, html)
 
 
 class WebUIHook(immp.ResourceHook):
@@ -128,7 +146,13 @@ class WebUIHook(immp.ResourceHook):
                    [(module, logging.getLevelName(logger.level))
                     for module, logger in sorted(logging.root.manager.loggerDict.items())
                     if isinstance(logger, logging.Logger) and logger.level != logging.NOTSET])
-        return {"loggers": loggers,
+        uptime = None
+        if self.host.started:
+            uptime = datetime.now() - self.host.started
+            # Drop microseconds from the delta (no datetime.replace equivalent for timedelta).
+            uptime = timedelta(days=uptime.days, seconds=uptime.seconds)
+        return {"uptime": uptime,
+                "loggers": loggers,
                 "versions": [("Python", self._python_version), ("IMMP", self._host_version)]}
 
     async def add(self, request):
@@ -145,10 +169,12 @@ class WebUIHook(immp.ResourceHook):
             raise web.HTTPNotFound
         if "schema" in post:
             config = post.get("config") or ""
+            doc, doc_html = _render_module_doc(cls)
             return {"path": path,
                     "config": config,
                     "class": cls,
-                    "doc": _module_doc(cls),
+                    "doc": doc,
+                    "doc_html": doc_html,
                     "hook": issubclass(cls, immp.Hook)}
         try:
             name = post["name"]
@@ -198,8 +224,10 @@ class WebUIHook(immp.ResourceHook):
         if source:
             title = await immp.Channel(plug, source).title()
             name = re.sub(r"[^a-z0-9]+", "-", title, flags=re.I).strip("-") if title else ""
+        doc, doc_html = _render_module_doc(plug.__class__)
         return {"plug": plug,
-                "doc": _module_doc(plug.__class__),
+                "doc": doc,
+                "doc_html": doc_html,
                 "add_name": name,
                 "add_source": source,
                 "channels": {name: channel for name, channel in self.host.channels.items()
@@ -217,12 +245,12 @@ class WebUIHook(immp.ResourceHook):
 
     async def plug_stop(self, request):
         plug = self._resolve_plug(request)
-        await plug.close()
+        ensure_future(plug.close())
         raise web.HTTPFound(self.ctx.url_for("plug", name=plug.name))
 
     async def plug_start(self, request):
         plug = self._resolve_plug(request)
-        await plug.open()
+        ensure_future(plug.open())
         raise web.HTTPFound(self.ctx.url_for("plug", name=plug.name))
 
     async def plug_config(self, request):
@@ -450,8 +478,10 @@ class WebUIHook(immp.ResourceHook):
     async def hook(self, request):
         hook = self._resolve_hook(request)
         can_stop = not isinstance(hook, (WebHook, WebUIHook))
+        doc, doc_html = _render_module_doc(hook)
         return {"hook": hook,
-                "doc": _module_doc(hook),
+                "doc": doc,
+                "doc_html": doc_html,
                 "resource": isinstance(hook, immp.ResourceHook),
                 "priority": self.host.priority.get(hook.name),
                 "can_stop": can_stop}
@@ -471,12 +501,12 @@ class WebUIHook(immp.ResourceHook):
         if isinstance(hook, (WebHook, WebUIHook)):
             # This will hang due to trying to serve this request at the same time.
             raise web.HTTPBadRequest
-        await hook.close()
+        ensure_future(hook.close())
         raise web.HTTPFound(self.hook_url_for(hook, None))
 
     async def hook_start(self, request):
         hook = self._resolve_hook(request)
-        await hook.open()
+        ensure_future(hook.open())
         raise web.HTTPFound(self.hook_url_for(hook, None))
 
     async def hook_config(self, request):
