@@ -25,6 +25,7 @@ This will generate a cookie file called *refresh_token.txt* in the current direc
 from asyncio import CancelledError, Condition, ensure_future, gather, sleep
 from copy import copy
 from io import BytesIO
+from itertools import chain
 import logging
 import re
 from urllib.parse import unquote
@@ -134,23 +135,25 @@ class HangoutsSegment(immp.Segment):
                    link=link)
 
     @classmethod
-    def _make_segment(cls, text, segment):
-        text = re.sub("^ ", "\N{NBSP}", text).replace("  ", " \N{NBSP}")
-        return hangups.ChatMessageSegment(text,
-                                          (hangouts_pb2.SEGMENT_TYPE_LINK
-                                           if segment.link else
-                                           hangouts_pb2.SEGMENT_TYPE_TEXT),
-                                          is_bold=segment.bold,
-                                          is_italic=segment.italic,
-                                          is_underline=segment.underline,
-                                          is_strikethrough=segment.strike,
-                                          link_target=segment.link)
+    def _make_segments(cls, segment):
+        text = re.sub("^ ", "\N{NBSP}", segment.text).replace("  ", " \N{NBSP}")
+        for i, line in enumerate(text.split("\n")):
+            if i:
+                yield hangups.ChatMessageSegment("\n", hangouts_pb2.SEGMENT_TYPE_LINE_BREAK)
+            yield hangups.ChatMessageSegment(line,
+                                             (hangouts_pb2.SEGMENT_TYPE_LINK
+                                              if segment.link else
+                                              hangouts_pb2.SEGMENT_TYPE_TEXT),
+                                             is_bold=segment.bold,
+                                             is_italic=segment.italic,
+                                             is_underline=segment.underline,
+                                             is_strikethrough=segment.strike,
+                                             link_target=segment.link)
 
     @classmethod
     def to_segments(cls, segment):
         """
         Convert a :class:`.Segment` into one or more :class:`hangups.ChatMessageSegment` instances.
-        This will also attempt to create missing link segments for plain URLs within the text.
 
         Args:
             segment (.Segment)
@@ -160,32 +163,16 @@ class HangoutsSegment(immp.Segment):
             hangups.ChatMessageSegment list:
                 Unparsed segment objects.
         """
-        unlinked = copy(segment)
-        unlinked.link = None
-        if segment.link:
-            if segment.text == segment.link:
-                return [cls._make_segment(segment.text, segment)]
-            else:
-                return [cls._make_segment("{} [".format(segment.text), unlinked),
-                        cls._make_segment(segment.link, segment),
-                        cls._make_segment("]", unlinked)]
-        segments = []
-        for i, line in enumerate(segment.text.split("\n")):
-            if i:
-                segments.append(hangups.ChatMessageSegment(
-                    "\n", hangouts_pb2.SEGMENT_TYPE_LINE_BREAK))
-            while True:
-                match = re.search(r"\b([a-z-]+://\S+)\b", line)
-                if not match:
-                    break
-                left, right = match.span()
-                linked = copy(segment)
-                linked.link = line[left:right]
-                segments += [cls._make_segment(line[:left], segment),
-                             cls._make_segment(linked.link, linked)]
-                line = line[right:]
-            segments.append(cls._make_segment(line, unlinked))
-        return [segment for segment in segments if segment.text]
+        if segment.link and not segment.text_is_link:
+            # The server rejects segments whose link targets don't look like their text, including
+            # any free text as well as masquerading of links (e.g. text "a.com" and link "b.org").
+            parts = [copy(segment) for _ in range(3)]
+            for part, text in zip(parts, ("{} [".format(segment.text), segment.link, "]")):
+                part.text = text
+            parts[0].link = parts[2].link = None
+        else:
+            parts = [segment]
+        return list(chain(*(cls._make_segments(part) for part in parts)))
 
 
 class HangoutsLocation(immp.Location):
@@ -485,7 +472,7 @@ class HangoutsPlug(immp.Plug, immp.HTTPOpenable):
     async def start(self):
         await super().start()
         self._closing = False
-        self._client = hangups.Client(hangups.get_auth_stdin(self.config["cookie"]))
+        self._client = hangups.Client(hangups.get_auth_stdin(self.config["cookie"], True))
         self._client.on_connect.add_observer(self._connect)
         log.debug("Connecting client")
         self._looped = ensure_future(self._loop())
@@ -600,6 +587,38 @@ class HangoutsPlug(immp.Plug, immp.HTTPOpenable):
             participant_id=hangouts_pb2.ParticipantId(gaia_id=user.id))
         await self._client.remove_user(request)
 
+    async def channel_link_create(self, channel, shared=True):
+        try:
+            conv = self._convs.get(channel.source)
+        except KeyError:
+            return None
+        # Hangouts has no concept of private invite links.
+        if not shared:
+            return None
+        # Enable joining via link for the conversation.
+        if not conv.is_group_link_sharing_enabled:
+            await conv.set_group_link_sharing_enabled(True)
+            log.debug("Enabled join-by-link for %r", channel.source)
+        # Request a new invite link (this won't revoke any existing ones).
+        request = hangouts_pb2.GetGroupConversationUrlRequest(
+            request_header=self._client.get_request_header(),
+            conversation_id=hangouts_pb2.ConversationId(id=channel.source))
+        response = await self._client.get_group_conversation_url(request)
+        return response.group_conversation_url
+
+    async def channel_link_revoke(self, channel, link=None):
+        # Hangouts has no concept of revoking links -- any previously issued links will continue
+        # to work forever.  Instead, just disable joining via link for the default revocation.
+        if link:
+            return
+        try:
+            conv = self._convs.get(channel.source)
+        except KeyError:
+            return
+        if conv.is_group_link_sharing_enabled:
+            await conv.set_group_link_sharing_enabled(False)
+            log.debug("Disabled join-by-link for %r", channel.source)
+
     async def _next_batch(self, conv, before_id):
         # Conversation.get_events() should, if the target is the oldest message in the current
         # batch, fetch the next whole batch and return that, or else return everything before the
@@ -692,11 +711,14 @@ class HangoutsPlug(immp.Plug, immp.HTTPOpenable):
         return hangouts_pb2.ExistingMedia(photo=hangouts_pb2.Photo(photo_id=photo))
 
     @classmethod
-    def _serialise(cls, segments):
+    def _serialise(cls, rich):
         output = []
-        for segment in segments:
-            output += HangoutsSegment.to_segments(segment)
-        return [segment.serialize() for segment in output]
+        for chunk in rich.chunked(4096):
+            line = []
+            for segment in chunk:
+                line += HangoutsSegment.to_segments(segment)
+            output.append([segment.serialize() for segment in line])
+        return output
 
     def _request(self, conv, segments=None, media=None, place=None):
         return hangouts_pb2.SendChatMessageRequest(
@@ -721,17 +743,18 @@ class HangoutsPlug(immp.Plug, immp.HTTPOpenable):
         requests = []
         if msg.text or msg.reply_to:
             render = msg.render(link_name=False, edit=msg.edited, quote_reply=True)
-            segments = self._serialise(render)
+            parts = self._serialise(render)
             media = None
-            if len(images) == 1:
+            if len(images) == 1 and len(parts) == 1:
                 # Attach the only image to the message text.
                 media = images.pop()
-            requests.append(self._request(conv, segments, media))
+            for segments in parts:
+                requests.append(self._request(conv, segments, media))
         if images:
             segments = []
             if msg.user:
                 label = immp.Message(user=msg.user, text="sent an image", action=True)
-                segments = self._serialise(label.render(link_name=False))
+                segments = self._serialise(label.render(link_name=False))[0]
             # Send any additional media items in their own separate messages.
             for media in images:
                 requests.append(self._request(conv, segments, media))
@@ -742,7 +765,7 @@ class HangoutsPlug(immp.Plug, immp.HTTPOpenable):
             # Include a label only if we haven't sent a text message earlier.
             if msg.user and not msg.text:
                 label = immp.Message(user=msg.user, text="sent a location", action=True)
-                segments = self._serialise(label.render(link_name=False))
+                segments = self._serialise(label.render(link_name=False))[0]
                 requests.append(self._request(conv, segments))
         return requests
 
