@@ -16,6 +16,9 @@ Config:
             Primary nick for the bot user.
         real-name (str):
             Real name, as displayed in WHO queries.
+    perform (str list):
+        List of raw IRC lines to send during the initial connection (after ``NICK`` and ``USER``).
+        Only applies to the main plug user, not puppets.
     quit (str):
         Quit message, sent as part of disconnection from the server.
     accept-invites (bool):
@@ -28,15 +31,23 @@ Config:
         be made when sending a message with an unseen username, and reused for later messages.
     puppet-prefix (str):
         Leading characters to include in nicks of puppet users.
+    send-delay (float):
+        Time in seconds to wait between sending each message (0.5 by default, i.e. 2 messages per
+        second), in order to avoid being kicked for flooding.
+
+        You can reduce this if the connecting IRC user has been given higher limits on the server,
+        or increase this for servers with stricter flood limits.
 
 Channel sources should include the correct channel prefix or prefixes (typically just ``#``) for
 shared channels, and bare IRC nicks for private channels.
 """
 
-from asyncio import (CancelledError, ensure_future, Event, Future, open_connection, sleep,
+from asyncio import (CancelledError, ensure_future, Event, Future, Lock, open_connection, sleep,
                      TimeoutError, wait_for)
 import codecs
+from datetime import datetime, timedelta
 from hashlib import md5
+from itertools import chain
 import logging
 import re
 
@@ -151,9 +162,8 @@ class IRCSegment(immp.Segment):
 
     @classmethod
     def _coloured(cls, colour, text):
-        # Includes a default background colour to avoid accidental combinations with a literal comma
-        # following the text.
-        return "\x03{}{}\x0399,99".format(colour, text)
+        # Beware of digits following coloured text -- see IRCRichText.to_formatted().
+        return "\x03{}{}\x03".format(colour, text)
 
     @classmethod
     def to_formatted(cls, segment):
@@ -176,9 +186,12 @@ class IRCSegment(immp.Segment):
         if segment.underline:
             text = "\x1f{}\x1f".format(text)
         if segment.strike:
-            # Muted text by colouring it grey.
+            # https://modern.ircdocs.horse/formatting.html#strikethrough -- 0x1e is proposed, but
+            # not standard (e.g. irssi ignores it and renders the control characters as inverted
+            # carets).  Instead, emulate with muted text by colouring it grey, which will hopefully
+            # look reasonable on both light and dark display modes of clients.
             text = cls._coloured(14, text)
-        if segment.link and segment.text != segment.link:
+        if segment.link and not segment.text_is_link:
             text = "{} [{}]".format(text, segment.link)
         return text
 
@@ -201,7 +214,19 @@ class IRCRichText(immp.RichText):
             str:
                 Code-formatted string.
         """
-        return "".join(IRCSegment.to_formatted(segment) for segment in rich).replace("\t", " ")
+        clone = rich.clone()
+        strike = False
+        for segment in clone:
+            if segment.strike:
+                strike = True
+            elif strike:
+                strike = False
+                if segment.text[:1].isdigit():
+                    # Reset-colour control code followed by a digit (e.g. `\x031`), which would be
+                    # inadvertently parsed as a replacement colour -- add an extra space between
+                    # the two segments.
+                    segment.text = " {}".format(segment.text)
+        return "".join(IRCSegment.to_formatted(segment) for segment in clone).replace("\t", " ")
 
 
 class IRCUser(immp.User):
@@ -269,6 +294,26 @@ class IRCUser(immp.User):
         real_name = line.args[-1].split(" ", 1)[-1]
         return immp.User(id_=id_, plug=irc, username=username, real_name=real_name, raw=line)
 
+    @classmethod
+    def from_whois(cls, irc, line):
+        """
+        Convert the response of a ``WHOIS`` query into a :class:`.User`.
+
+        Args:
+            irc (.IRCPlug):
+                Related plug instance that provides the user.
+            line (.Line):
+                311-numeric line containing a user's nick, host and real name.
+
+        Returns:
+            .User:
+                Parsed user object.
+        """
+        id_ = "{}!{}@{}".format(line.args[1], line.args[2], line.args[3])
+        username = line.args[1]
+        real_name = line.args[-1]
+        return immp.User(id_=id_, plug=irc, username=username, real_name=real_name, raw=line)
+
 
 class IRCMessage(immp.Message):
     """
@@ -377,6 +422,8 @@ class Wait:
             line (.Line):
                 Incoming line to be handled.
         """
+        if self.done:
+            raise ValueError("Wait already resolved")
         if line.command == IRCTryAgain.COMMAND:
             self._result.set_exception(IRCTryAgain(line))
             return
@@ -430,7 +477,8 @@ class IRCClient:
         self._host = host
         self._port = port
         self._ssl = ssl
-        self._nick = self._nick_bad_chars.sub("", nick)
+        self._nick_target = self._nick = self._nick_bad_chars.sub("", nick)
+        self._nick_tried = datetime.now()
         self._password = password
         self._user = user
         self._name = name
@@ -467,11 +515,12 @@ class IRCClient:
         value = self._nick_bad_chars.sub("", value)
         if self._nicklen:
             value = value[:self._nicklen]
+        self._nick_target = value
         if self._nick == value:
             return
         if self._writer:
             await self._wait(Line("NICK", value),
-                             success=("001", "NICK"),
+                             success=("NICK",),
                              fail=("431", "432", "433", "436"))
         else:
             self._nick = value
@@ -499,6 +548,25 @@ class IRCClient:
     def closing(self):
         return self._closing
 
+    async def _regain_nick(self):
+        if self._nick_target == self._nick:
+            return
+        # Don't bother if we can actually see someone using our nick.  Event handling should update
+        # the member cache to match before calling this function.  Don't check the user cache as
+        # it might include dropped users that weren't in a mutual channel at the time.
+        if any(self._nick_target in members for members in self.members.values()):
+            return
+        # Wait at least 30 seconds before trying again.
+        now = datetime.now()
+        if self._nick_tried + timedelta(seconds=30) > now:
+            return
+        self._nick_tried = now
+        log.debug("Client %r attempting to regain nick %r", self._nick, self._nick_target)
+        try:
+            await self.set_nick(self._nick_target)
+        except IRCError as e:
+            log.debug("Client %r failed to regain nick (%s)", self._nick, e.args[0].command)
+
     async def _read_loop(self):
         while True:
             try:
@@ -537,6 +605,7 @@ class IRCClient:
                     break
             finally:
                 self._live.clear()
+            await self._regain_nick()
         if not self._closing:
             ensure_future(self._reconnect("Disconnected"))
 
@@ -566,7 +635,7 @@ class IRCClient:
         for wait in self._waits:
             try:
                 wait.add(line)
-            except TypeError:
+            except (TypeError, ValueError):
                 continue
             else:
                 break
@@ -594,7 +663,7 @@ class IRCClient:
                     self.network = value
         elif line.command in ("431", "432"):
             # Nick change rejected, revert internal nick to who the server says we are.
-            if self._nick != line.args[0]:
+            if line.args[0] not in (self._nick, "*"):
                 log.debug("Reverting failed nick change: %s -> %s", self._nick, line.args[0])
                 self._nick = line.args[0]
         elif line.command in ("433", "436"):
@@ -602,17 +671,6 @@ class IRCClient:
             # Remove characters from the nick if needed to make it fit.
             parsed = line.args[1]
             log.debug("Nick collision: %s", parsed)
-            if len(parsed) < len(self._nick):
-                # We got silently truncated, set the max nick length.
-                self._nicklen = len(parsed)
-                self._nick = parsed
-            if len(self._nick.rstrip("_")) < 2:
-                raise ValueError("Nick exhausted")
-            if self._nicklen and len(self._nick) >= self._nicklen:
-                base = self._nick[:self._nicklen].rstrip("_")
-                self.nick = base[:-1].ljust(self._nicklen, "_")
-            else:
-                self.nick += "_"
         elif line.command == "PING":
             self._write(Line("PONG", *line.args))
         elif line.command in ("JOIN", "PART", "KICK", "PRIVMSG"):
@@ -634,12 +692,20 @@ class IRCClient:
                 elif nick in members:
                     log.debug("Converting QUIT to PART for %s in %s", nick, name)
                     await self._handle(Line("PART", name, source=line.source))
+            if self._nick_target == nick:
+                # Someone holding the nick we want just disconnected.
+                ensure_future(self._regain_nick())
         elif line.command == "NICK":
             old = line.source.split("!", 1)[0]
             new = line.args[0]
-            if self._nick == old:
-                log.debug("Updating own nick: %s -> %s", self._nick, new)
-                self._nick = new
+            # Sync user and member caches.
+            try:
+                user = self.users.pop(old)
+            except KeyError:
+                pass
+            else:
+                user.username = new
+                self.users[new] = user
             for name, members in list(self.members.items()):
                 if name == old:
                     log.debug("Replacing %s with %s in self entry", old, new)
@@ -649,6 +715,18 @@ class IRCClient:
                     log.debug("Replacing %s with %s in %s members", old, new, name)
                     members.remove(old)
                     members.add(new)
+            # Update our own nick if needed.
+            if self._nick == old:
+                if len(new) < len(old) and old.startswith(new):
+                    # We got silently truncated, set the max nick length.
+                    self._nicklen = len(new)
+                log.debug("Updating own nick: %s -> %s", self._nick, new)
+                self._nick = new
+                # We might have been renamed away from the nick we want.
+                ensure_future(self._regain_nick())
+            elif self._nick_target == old:
+                # Someone just released the nick we want.
+                ensure_future(self._regain_nick())
         if self._on_receive:
             await self._on_receive(line)
 
@@ -657,16 +735,31 @@ class IRCClient:
         Join the target IRC server.
         """
         self._closing = False
+        self._nick = self._nick_target
         self._reader, self._writer = await open_connection(self._host, self._port, ssl=self._ssl)
         self._read_task = ensure_future(self._read_loop())
         if self._password:
             self._write(Line("PASS", self._password))
-        await self._wait(Line("USER", self._user, "0", "*", self._name),
-                         Line("NICK", self._nick),
-                         success=("001",))
-        for user in await self.who(self._nick):
-            if user.username == self._nick:
-                self._mask = user.id.split("!", 1)[-1]
+        self._write(Line("USER", self._user, "0", "*", self._name))
+        while True:
+            try:
+                await self._wait(Line("NICK", self._nick),
+                                 success=("001",), fail=("431", "432", "433", "436"))
+            except IRCError as e:
+                if len(self._nick.rstrip("_")) < 2:
+                    raise ValueError("Nick options exhausted")
+                elif e.args[0].command in ("431", "432"):
+                    raise
+                if self._nicklen and len(self._nick) >= self._nicklen:
+                    base = self._nick[:self._nicklen].rstrip("_")
+                    self._nick = base[:-1].ljust(self._nicklen, "_")
+                else:
+                    self._nick += "_"
+            else:
+                break
+        user = await self.whois(self._nick)
+        if user:
+            self._mask = user.id.split("!", 1)[-1]
         self._live_task = ensure_future(self._keepalive_loop())
         if self._on_connect:
             await self._on_connect()
@@ -705,23 +798,35 @@ class IRCClient:
 
     async def _reconnect(self, msg):
         await self.disconnect(msg)
-        log.debug("Reconnecting in 3 seconds")
-        await sleep(3)
-        await self.connect()
+        delay = 3
+        while True:
+            log.debug("Client %r reconnecting in %d seconds", self._nick, delay)
+            await sleep(delay)
+            try:
+                await self.connect()
+            except Exception:
+                log.warning("Client %r reconnect to %r failed",
+                            self._nick, self._host, exc_info=True)
+                delay = min(delay * 2, 30)
+            else:
+                log.debug("Client %r reconnect to %r successful", self._nick, self._host)
+                return
 
     async def who(self, name):
         """
-        Perform a user lookup on the server.
+        Perform a member lookup on the server.
 
         Args:
             name (str):
                 User nick or channel name.
 
         Returns:
-            .Line list:
-                Response lines from the server.
+            .User set:
+                Matching users, either a single user or all participants of a channel.
         """
-        if name in self.users:
+        if name in self.members:
+            return self.members[name]
+        elif name in self.users:
             self.members[name] = {name}
             return {self.users[name]}
         members = set()
@@ -738,6 +843,28 @@ class IRCClient:
             del self.members[name]
         return users
 
+    async def whois(self, name):
+        """
+        Perform a user lookup on the server.
+
+        Args:
+            name (str):
+                User nick.
+
+        Returns:
+            .User:
+                Matching user, or ``None`` if not found.
+        """
+        if name in self.users:
+            return self.users[name]
+        for line in await self._wait(Line("WHOIS", name), success=("318",), collect=("311",)):
+            user = IRCUser.from_whois(self._plug, line)
+            self.users[name] = user
+            self.members[name] = {name}
+            return user
+        else:
+            return None
+
     async def join(self, channel):
         """
         Attempt to join the given channel.
@@ -748,10 +875,24 @@ class IRCClient:
         """
         if channel in self.members and self._nick in self.members[channel]:
             return
+        await self._regain_nick()
         await self._wait(Line("JOIN", channel), success=("JOIN",))
         await self.who(channel)
 
-    def invite(self, channel, nick):
+    async def part(self, channel):
+        """
+        Leave a channel you're participating in.
+
+        Args:
+            channel str):
+                Target channel name.
+        """
+        if self._nick not in self.members.get(channel, ()):
+            return
+        await self._regain_nick()
+        await self._wait(Line("PART", channel), success=("PART",))
+
+    async def invite(self, channel, nick):
         """
         Invite another user to a channel you're partipating in.
 
@@ -761,9 +902,10 @@ class IRCClient:
             nick (str):
                 User to be invited.
         """
+        await self._regain_nick()
         self._write(Line("INVITE", nick, channel))
 
-    def kick(self, channel, nick):
+    async def kick(self, channel, nick):
         """
         Remove a user from a channel you're an operator of.
 
@@ -773,6 +915,7 @@ class IRCClient:
             nick (str):
                 User to be kicked.
         """
+        await self._regain_nick()
         self._write(Line("KICK", channel, nick))
 
     async def list(self):
@@ -795,7 +938,7 @@ class IRCClient:
         """
         return await self._wait(Line("NAMES"), success=("366",), fail=("401",), collect=("353",))
 
-    def send(self, channel, text):
+    async def send(self, channel, text):
         """
         Send a message to a user or channel.
 
@@ -809,6 +952,7 @@ class IRCClient:
             .Line:
                 Resulting line sent to the server.
         """
+        await self._regain_nick()
         line = Line("PRIVMSG", channel, text)
         self._write(line)
         line.source = self.nickmask
@@ -818,6 +962,27 @@ class IRCClient:
         return "<{}: {!r}>".format(self.__class__.__name__, self.nickmask or self.nick)
 
 
+class DelayLock:
+    """
+    Timed-release lock, used in an ``async with`` statement to acquire a lock, but delay its
+    release by some number of seconds after the caller has finished with it.
+    """
+
+    def __init__(self, delay):
+        self._delay = delay
+        self._lock = Lock()
+
+    async def _delay_release(self):
+        await sleep(self._delay())
+        self._lock.release()
+
+    async def __aenter__(self):
+        await self._lock.acquire()
+
+    async def __aexit__(self, exc_type, exc_value, traceback):
+        ensure_future(self._delay_release())
+
+
 class IRCPlug(immp.Plug):
     """
     Plug for an IRC server.
@@ -829,11 +994,13 @@ class IRCPlug(immp.Plug):
                                      immp.Optional("password"): immp.Nullable(str)},
                           "user": {"nick": str,
                                    "real-name": str},
+                          immp.Optional("perform", list): [str],
                           immp.Optional("quit", "Disconnecting"): str,
                           immp.Optional("accept-invites", False): bool,
                           immp.Optional("colour-nicks", False): bool,
                           immp.Optional("puppet", False): bool,
-                          immp.Optional("puppet-prefix", ""): str})
+                          immp.Optional("puppet-prefix", ""): str,
+                          immp.Optional("send-delay", 0.5): float})
 
     def __init__(self, name, config, host):
         super().__init__(name, config, host)
@@ -842,6 +1009,8 @@ class IRCPlug(immp.Plug):
         self._joins = set()
         # Maintain puppet clients by nick for cleaner sending.
         self._puppets = {}
+        # Queue multiple outgoing messages in quick succession and insert delays between them.
+        self._delay_lock = DelayLock(lambda: self.config["send-delay"])
 
     @property
     def network_name(self):
@@ -893,11 +1062,8 @@ class IRCPlug(immp.Plug):
         for client in self._puppets.values():
             if username == client.nick:
                 return IRCUser.from_id(self, client.nickmask, client.name)
-        for user in await self._client.who(username):
-            if user.username == username:
-                return user
         else:
-            return None
+            return await self._client.whois(username)
 
     async def user_is_system(self, user):
         if user.id == self._client.nickmask:
@@ -921,13 +1087,13 @@ class IRCPlug(immp.Plug):
             raw = await self._client.names()
         except IRCTryAgain:
             return None
-        names = set()
+        names = set(self._client.users)
         for line in raw:
             names.update(name.lstrip(self._client.prefixes) for name in line.args[3].split())
+        names.discard(self._client.nick)
         for client in self._puppets.values():
-            names.remove(client.nick)
-        return [immp.Channel(self, name) for name in names
-                if name != self.config["user"]["nick"]]
+            names.discard(client.nick)
+        return [immp.Channel(self, name) for name in names]
 
     async def channel_for_user(self, user):
         return immp.Channel(self, user.username)
@@ -952,12 +1118,14 @@ class IRCPlug(immp.Plug):
         return members
 
     async def channel_invite(self, channel, user):
-        self._client.invite(channel.source, user.username)
+        await self._client.invite(channel.source, user.username)
 
     async def channel_remove(self, channel, user):
-        self._client.kick(channel.source, user.username)
+        await self._client.kick(channel.source, user.username)
 
     async def _connected(self):
+        for perform in self.config["perform"]:
+            self._client._write(Line.parse(perform))
         for channel in self.host.channels.values():
             if channel.plug == self and channel.source.startswith("#"):
                 self._joins.add(channel.source)
@@ -986,34 +1154,58 @@ class IRCPlug(immp.Plug):
         elif line.command == "INVITE" and self.config["accept-invites"]:
             await self._client.join(line.args[1])
 
-    def _lines(self, rich, user, action, edited, plain=False):
+    def _inline(self, rich):
+        # Take an excerpt of some message text, merging multiple lines into one.
+        if not rich:
+            return rich
+        inlined = rich.clone()
+        for segment in inlined:
+            segment.text = segment.text.replace("\n", "  ")
+        return inlined.trim(160)
+
+    def _author_name(self, user):
+        name = user.username or user.real_name
+        if self.config["colour-nicks"]:
+            name = IRCSegment._coloured(IRCUser.nick_colour(name), name)
+        return name
+
+    def _author_template(self, user=None, action=False, edited=False, quoter=None):
+        prefix = []
+        suffix = []
+        if user:
+            prefix.append(("* {} " if action else "<{}> ").format(self._author_name(user)))
+        if quoter:
+            prefix.append("<{}> [".format(self._author_name(quoter)))
+            suffix.append("]")
+        if edited:
+            prefix.append("[edit] ")
+        if action and not user and not quoter:
+            prefix.append("\x01ACTION ")
+            suffix.append("\x01")
+        return "{}{{}}{}".format("".join(reversed(prefix)), "".join(suffix)).strip()
+
+    def _lines(self, rich, user=None, action=False, edited=False, quoter=None):
         if not rich:
             return []
         elif not isinstance(rich, immp.RichText):
             rich = immp.RichText([immp.Segment(rich)])
+        template = self._author_template(user, action, edited, quoter)
         lines = []
-        for text in IRCRichText.to_formatted(rich).split("\n"):
-            if user:
-                template = "* {} {}" if action else "<{}> {}"
-                name = user.username or user.real_name
-                if self.config["colour-nicks"]:
-                    name = IRCSegment._coloured(IRCUser.nick_colour(name), name)
-                text = template.format(name, text)
-            if edited:
-                text = "[edit] {}".format(text)
-            if not user and action:
-                template = "* {}" if plain else "\x01ACTION {}\x01"
-                text = template.format(text)
-            lines.append(text)
+        # Line length isn't well defined (generally 512 bytes for the entire wire line), so set a
+        # conservative length limit to allow for long channel names and formatting characters.
+        for line in chain(*(chunk.lines() for chunk in rich.chunked(360))):
+            text = IRCRichText.to_formatted(line)
+            lines.append(template.format(text))
         return lines
 
-    async def _puppet(self, user):
+    async def _puppet(self, user, create=True):
         username = user.username or user.real_name
         nick = self.config["puppet-prefix"] + "-".join(username.split())
         try:
             puppet = self._puppets[user]
         except KeyError:
-            pass
+            if not create:
+                return None
         else:
             log.debug("Reusing puppet %r for user %r", puppet, user)
             if puppet.nick.rstrip("_") != nick:
@@ -1044,12 +1236,9 @@ class IRCPlug(immp.Plug):
     async def put(self, channel, msg):
         user = None if self.config["puppet"] else msg.user
         lines = []
-        if isinstance(msg.reply_to, immp.Message):
-            quote = self._lines(msg.reply_to.text, msg.reply_to.user,
-                                msg.reply_to.action, msg.edited, True)
-            if quote:
-                snippet = "{}{}".format(quote[0], " [...]" if len(quote) > 1 else "")
-                lines += self._lines(snippet, user, False, msg.edited)
+        if isinstance(msg.reply_to, immp.Message) and msg.reply_to.text:
+            lines.append(self._lines(self._inline(msg.reply_to.text), msg.reply_to.user,
+                                     msg.reply_to.action, msg.edited, user)[0])
         if msg.text:
             lines += self._lines(msg.text, user, msg.action, msg.edited)
         for attach in msg.attachments:
@@ -1060,9 +1249,7 @@ class IRCPlug(immp.Plug):
                 text = "shared a location: {}".format(attach)
                 lines += self._lines(text, user, True, msg.edited)
             elif isinstance(attach, immp.Message) and attach.text:
-                forward = self._lines(attach.text, attach.user, attach.action, attach.edited, True)
-                if forward:
-                    lines += self._lines("\n".join(forward), user, False, msg.edited)
+                lines += self._lines(attach.text, attach.user, attach.action, attach.edited, user)
         receipts = []
         if self.config["puppet"] and msg.user:
             client = await self._puppet(msg.user)
@@ -1071,8 +1258,18 @@ class IRCPlug(immp.Plug):
         else:
             client = self._client
         for text in lines:
-            line = client.send(channel.source, text)
+            async with self._delay_lock:
+                line = await client.send(channel.source, text)
             sent = await IRCMessage.from_line(self, line)
             self.queue(sent)
             receipts.append(sent)
+        if self.config["puppet"]:
+            for member in msg.joined:
+                puppet = await self._puppet(member, False)
+                if puppet:
+                    ensure_future(puppet.join(channel.source))
+            for member in msg.left:
+                puppet = await self._puppet(member, False)
+                if puppet:
+                    ensure_future(puppet.part(channel.source))
         return receipts
