@@ -142,7 +142,8 @@ class _Schema:
                  immp.Optional("hidden", False): bool,
                  immp.Optional("channel"): immp.Nullable(str),
                  immp.Optional("edited", dict):
-                     {immp.Optional("user"): immp.Nullable(str)},
+                     {immp.Optional("ts"): immp.Nullable(str),
+                      immp.Optional("user"): immp.Nullable(str)},
                  immp.Optional("thread_ts"): immp.Nullable(str),
                  immp.Optional("files", list): [file],
                  immp.Optional("attachments", list): [attachment],
@@ -476,12 +477,18 @@ class SlackRichText(immp.RichText):
                 if getattr(segment, attr) and tag not in active:
                     text += tag
                     active.append(tag)
-            if segment.mention and slack.same_team(segment.mention.plug):
-                text += "<@{}>".format(segment.mention.id)
-            elif segment.link:
-                text += "<{}|{}>".format(segment.link, cls._escape(segment.text))
-            else:
-                text += cls._escape(segment.text)
+            parsed = cls._escape(segment.text)
+            if not segment.code and not segment.pre:
+                link = None
+                if not segment.mention:
+                    link = segment.link
+                elif slack.same_team(segment.mention.plug):
+                    parsed = "<@{}>".format(segment.mention.id)
+                else:
+                    link = segment.mention.link
+                if link:
+                    parsed = "<{}|{}>".format(link, cls._escape(segment.text))
+            text += parsed
         for tag in reversed(active):
             # Close all remaining tags.
             text += tag
@@ -597,8 +604,9 @@ class SlackMessage(immp.Message):
         return user
 
     @classmethod
-    async def _parse_main(cls, slack, json, event, channel, parent=True, revision=None):
+    async def _parse_main(cls, slack, json, event, channel, parent=True):
         id_, at = cls._parse_meta(slack, event)
+        revision = event["edited"]["ts"]
         edited = bool(revision)
         deleted = False
         text = event["text"]
@@ -717,8 +725,8 @@ class SlackMessage(immp.Message):
                 Parsed message object.
         """
         event = _Schema.message(json)
-        if event["hidden"]:
-            # Ignore UI-hidden events (e.g. tombstones of deleted files).
+        if event["hidden"] and event["subtype"] != "message_changed":
+            # Ignore most UI-hidden events (e.g. tombstones of deleted files).
             raise NotImplementedError("hidden")
         if event["is_ephemeral"]:
             # Ignore user-private messages from Slack (e.g. over quota warnings, link unfurling
@@ -738,15 +746,14 @@ class SlackMessage(immp.Message):
                                     raw=json)
         elif event["subtype"] == "message_changed":
             if event["message"]["hidden"]:
-                # In theory this should match event["hidden"], but redefined here just in case.
+                # We might get updates to messages that are themselves hidden.
                 raise NotImplementedError("hidden")
             if event["message"]["text"] == event["previous_message"]["text"]:
                 # Message remains unchanged.  Can be caused by link unfurling (adds an attachment)
                 # or deleting replies (reply is removed from event.replies in new *and old*).
                 raise NotImplementedError("unchanged")
-            revision = event["ts"]
             # Original message details are under a nested "message" key.
-            return await cls._parse_main(slack, json, event["message"], channel, parent, revision)
+            return await cls._parse_main(slack, json, event["message"], channel, parent)
         else:
             return await cls._parse_main(slack, json, event, channel, parent)
 
@@ -1116,7 +1123,11 @@ class SlackPlug(immp.Plug, immp.HTTPOpenable):
                 for segment in rich:
                     segment.italic = True
         if msg.edited:
-            rich.append(immp.Segment(" (edited)", italic=True))
+            if rich:
+                rich.append(immp.Segment(" "))
+            else:
+                rich = immp.RichText()
+            rich.append(immp.Segment("(edited)", italic=True))
         attachments = []
         if isinstance(parent.reply_to, immp.Receipt):
             data["thread_ts"] = msg.reply_to.id
@@ -1134,12 +1145,25 @@ class SlackPlug(immp.Plug, immp.HTTPOpenable):
                                     "text": attach.address,
                                     "footer": "{}, {}".format(attach.latitude, attach.longitude)})
         if rich or attachments:
+            chunks = []
             if rich:
-                data["text"] = SlackRichText.to_mrkdwn(self, rich)
+                text = SlackRichText.to_mrkdwn(self, rich)
+                chunks = immp.RichText.chunked_plain(text, 4000)
+            items = []
             if attachments:
-                data["attachments"] = json_dumps(attachments)
-            post = await self._api("chat.postMessage", _Schema.chat_post, data=data)
-            receipts.append(await SlackMessage.from_post(self, post))
+                item = dict(data)
+                if len(chunks) == 1:
+                    # Attach the only embed to the message text.
+                    item["text"] = chunks.pop()
+                item["attachments"] = json_dumps(attachments)
+                items.append(item)
+            for chunk in chunks:
+                item = dict(data)
+                item["text"] = chunk
+                items.append(item)
+            for item in items:
+                post = await self._api("chat.postMessage", _Schema.chat_post, data=item)
+                receipts.append(await SlackMessage.from_post(self, post))
         return receipts
 
     async def put(self, channel, msg):
@@ -1190,7 +1214,7 @@ class SlackPlug(immp.Plug, immp.HTTPOpenable):
                 if "envelope_id" in wrapper:
                     await self._socket.send_json({"envelope_id": wrapper["envelope_id"]})
                 if wrapper["type"] != "events_api" or not wrapper["payload"]:
-                    log.debug("User %r ignpring unknown Socket Mode event %r",
+                    log.debug("User %r ignoring unknown Socket Mode event %r",
                               self._bot_user, wrapper["type"])
                     continue
                 payload = wrapper["payload"]
